/****************************************************************************
** Meta object code from reading C++ file 'window.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.5.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/tanglet-1.4.0/src/window.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'window.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.5.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_Window_t {
    QByteArrayData data[24];
    char stringdata0[248];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Window_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Window_t qt_meta_stringdata_Window = {
    {
QT_MOC_LITERAL(0, 0, 6), // "Window"
QT_MOC_LITERAL(1, 7, 5), // "about"
QT_MOC_LITERAL(2, 13, 0), // ""
QT_MOC_LITERAL(3, 14, 11), // "aboutHspell"
QT_MOC_LITERAL(4, 26, 10), // "aboutScowl"
QT_MOC_LITERAL(5, 37, 7), // "newRoll"
QT_MOC_LITERAL(6, 45, 7), // "newGame"
QT_MOC_LITERAL(7, 53, 10), // "chooseGame"
QT_MOC_LITERAL(8, 64, 9), // "shareGame"
QT_MOC_LITERAL(9, 74, 7), // "endGame"
QT_MOC_LITERAL(10, 82, 9), // "autoPause"
QT_MOC_LITERAL(11, 92, 10), // "autoResume"
QT_MOC_LITERAL(12, 103, 9), // "setPaused"
QT_MOC_LITERAL(13, 113, 6), // "paused"
QT_MOC_LITERAL(14, 120, 11), // "showDetails"
QT_MOC_LITERAL(15, 132, 10), // "showScores"
QT_MOC_LITERAL(16, 143, 12), // "showLanguage"
QT_MOC_LITERAL(17, 156, 10), // "showLocale"
QT_MOC_LITERAL(18, 167, 12), // "showControls"
QT_MOC_LITERAL(19, 180, 17), // "optimizingStarted"
QT_MOC_LITERAL(20, 198, 18), // "optimizingFinished"
QT_MOC_LITERAL(21, 217, 11), // "gameStarted"
QT_MOC_LITERAL(22, 229, 12), // "gameFinished"
QT_MOC_LITERAL(23, 242, 5) // "score"

    },
    "Window\0about\0\0aboutHspell\0aboutScowl\0"
    "newRoll\0newGame\0chooseGame\0shareGame\0"
    "endGame\0autoPause\0autoResume\0setPaused\0"
    "paused\0showDetails\0showScores\0"
    "showLanguage\0showLocale\0showControls\0"
    "optimizingStarted\0optimizingFinished\0"
    "gameStarted\0gameFinished\0score"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Window[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      20,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  114,    2, 0x08 /* Private */,
       3,    0,  115,    2, 0x08 /* Private */,
       4,    0,  116,    2, 0x08 /* Private */,
       5,    0,  117,    2, 0x08 /* Private */,
       6,    0,  118,    2, 0x08 /* Private */,
       7,    0,  119,    2, 0x08 /* Private */,
       8,    0,  120,    2, 0x08 /* Private */,
       9,    0,  121,    2, 0x08 /* Private */,
      10,    0,  122,    2, 0x08 /* Private */,
      11,    0,  123,    2, 0x08 /* Private */,
      12,    1,  124,    2, 0x08 /* Private */,
      14,    0,  127,    2, 0x08 /* Private */,
      15,    0,  128,    2, 0x08 /* Private */,
      16,    0,  129,    2, 0x08 /* Private */,
      17,    0,  130,    2, 0x08 /* Private */,
      18,    0,  131,    2, 0x08 /* Private */,
      19,    0,  132,    2, 0x08 /* Private */,
      20,    0,  133,    2, 0x08 /* Private */,
      21,    0,  134,    2, 0x08 /* Private */,
      22,    1,  135,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   13,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   23,

       0        // eod
};

void Window::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Window *_t = static_cast<Window *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->about(); break;
        case 1: _t->aboutHspell(); break;
        case 2: _t->aboutScowl(); break;
        case 3: _t->newRoll(); break;
        case 4: _t->newGame(); break;
        case 5: _t->chooseGame(); break;
        case 6: _t->shareGame(); break;
        case 7: { bool _r = _t->endGame();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 8: _t->autoPause(); break;
        case 9: _t->autoResume(); break;
        case 10: _t->setPaused((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->showDetails(); break;
        case 12: _t->showScores(); break;
        case 13: _t->showLanguage(); break;
        case 14: _t->showLocale(); break;
        case 15: _t->showControls(); break;
        case 16: _t->optimizingStarted(); break;
        case 17: _t->optimizingFinished(); break;
        case 18: _t->gameStarted(); break;
        case 19: _t->gameFinished((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject Window::staticMetaObject = {
    { &QMainWindow::staticMetaObject, qt_meta_stringdata_Window.data,
      qt_meta_data_Window,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *Window::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Window::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_Window.stringdata0))
        return static_cast<void*>(const_cast< Window*>(this));
    return QMainWindow::qt_metacast(_clname);
}

int Window::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 20;
    }
    return _id;
}
QT_END_MOC_NAMESPACE
