/****************************************************************************
** Meta object code from reading C++ file 'language_dialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.5.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/tanglet-1.4.0/src/language_dialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'language_dialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.5.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_LanguageDialog_t {
    QByteArrayData data[14];
    char stringdata0[133];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LanguageDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LanguageDialog_t qt_meta_stringdata_LanguageDialog = {
    {
QT_MOC_LITERAL(0, 0, 14), // "LanguageDialog"
QT_MOC_LITERAL(1, 15, 6), // "accept"
QT_MOC_LITERAL(2, 22, 0), // ""
QT_MOC_LITERAL(3, 23, 7), // "clicked"
QT_MOC_LITERAL(4, 31, 16), // "QAbstractButton*"
QT_MOC_LITERAL(5, 48, 6), // "button"
QT_MOC_LITERAL(6, 55, 14), // "chooseLanguage"
QT_MOC_LITERAL(7, 70, 5), // "index"
QT_MOC_LITERAL(8, 76, 10), // "browseDice"
QT_MOC_LITERAL(9, 87, 10), // "chooseDice"
QT_MOC_LITERAL(10, 98, 4), // "dice"
QT_MOC_LITERAL(11, 103, 11), // "browseWords"
QT_MOC_LITERAL(12, 115, 11), // "chooseWords"
QT_MOC_LITERAL(13, 127, 5) // "words"

    },
    "LanguageDialog\0accept\0\0clicked\0"
    "QAbstractButton*\0button\0chooseLanguage\0"
    "index\0browseDice\0chooseDice\0dice\0"
    "browseWords\0chooseWords\0words"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LanguageDialog[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x0a /* Public */,
       3,    1,   50,    2, 0x08 /* Private */,
       6,    1,   53,    2, 0x08 /* Private */,
       8,    0,   56,    2, 0x08 /* Private */,
       9,    1,   57,    2, 0x08 /* Private */,
      11,    0,   60,    2, 0x08 /* Private */,
      12,    1,   61,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   13,

       0        // eod
};

void LanguageDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        LanguageDialog *_t = static_cast<LanguageDialog *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->accept(); break;
        case 1: _t->clicked((*reinterpret_cast< QAbstractButton*(*)>(_a[1]))); break;
        case 2: _t->chooseLanguage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->browseDice(); break;
        case 4: _t->chooseDice((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->browseWords(); break;
        case 6: _t->chooseWords((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject LanguageDialog::staticMetaObject = {
    { &QDialog::staticMetaObject, qt_meta_stringdata_LanguageDialog.data,
      qt_meta_data_LanguageDialog,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *LanguageDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LanguageDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_LanguageDialog.stringdata0))
        return static_cast<void*>(const_cast< LanguageDialog*>(this));
    return QDialog::qt_metacast(_clname);
}

int LanguageDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}
QT_END_MOC_NAMESPACE
