/****************************************************************************
** Meta object code from reading C++ file 'board.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.5.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/tanglet-1.4.0/src/board.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'board.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.5.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_Board_t {
    QByteArrayData data[25];
    char stringdata0[288];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Board_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Board_t qt_meta_stringdata_Board = {
    {
QT_MOC_LITERAL(0, 0, 5), // "Board"
QT_MOC_LITERAL(1, 6, 7), // "started"
QT_MOC_LITERAL(2, 14, 0), // ""
QT_MOC_LITERAL(3, 15, 8), // "finished"
QT_MOC_LITERAL(4, 24, 5), // "score"
QT_MOC_LITERAL(5, 30, 17), // "optimizingStarted"
QT_MOC_LITERAL(6, 48, 18), // "optimizingFinished"
QT_MOC_LITERAL(7, 67, 14), // "pauseAvailable"
QT_MOC_LITERAL(8, 82, 9), // "available"
QT_MOC_LITERAL(9, 92, 18), // "setShowMissedWords"
QT_MOC_LITERAL(10, 111, 4), // "show"
QT_MOC_LITERAL(11, 116, 19), // "setShowMaximumScore"
QT_MOC_LITERAL(12, 136, 8), // "QAction*"
QT_MOC_LITERAL(13, 145, 17), // "setShowWordCounts"
QT_MOC_LITERAL(14, 163, 16), // "updateScoreColor"
QT_MOC_LITERAL(15, 180, 11), // "gameStarted"
QT_MOC_LITERAL(16, 192, 10), // "clearGuess"
QT_MOC_LITERAL(17, 203, 5), // "guess"
QT_MOC_LITERAL(18, 209, 12), // "guessChanged"
QT_MOC_LITERAL(19, 222, 6), // "finish"
QT_MOC_LITERAL(20, 229, 12), // "wordSelected"
QT_MOC_LITERAL(21, 242, 13), // "letterClicked"
QT_MOC_LITERAL(22, 256, 7), // "Letter*"
QT_MOC_LITERAL(23, 264, 6), // "letter"
QT_MOC_LITERAL(24, 271, 16) // "showMaximumWords"

    },
    "Board\0started\0\0finished\0score\0"
    "optimizingStarted\0optimizingFinished\0"
    "pauseAvailable\0available\0setShowMissedWords\0"
    "show\0setShowMaximumScore\0QAction*\0"
    "setShowWordCounts\0updateScoreColor\0"
    "gameStarted\0clearGuess\0guess\0guessChanged\0"
    "finish\0wordSelected\0letterClicked\0"
    "Letter*\0letter\0showMaximumWords"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Board[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   99,    2, 0x06 /* Public */,
       3,    1,  100,    2, 0x06 /* Public */,
       5,    0,  103,    2, 0x06 /* Public */,
       6,    0,  104,    2, 0x06 /* Public */,
       7,    1,  105,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    1,  108,    2, 0x0a /* Public */,
      11,    1,  111,    2, 0x0a /* Public */,
      13,    1,  114,    2, 0x0a /* Public */,
      14,    0,  117,    2, 0x0a /* Public */,
      15,    0,  118,    2, 0x08 /* Private */,
      16,    0,  119,    2, 0x08 /* Private */,
      17,    0,  120,    2, 0x08 /* Private */,
      18,    0,  121,    2, 0x08 /* Private */,
      19,    0,  122,    2, 0x08 /* Private */,
      20,    0,  123,    2, 0x08 /* Private */,
      21,    1,  124,    2, 0x08 /* Private */,
      24,    0,  127,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    8,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, 0x80000000 | 12,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 22,   23,
    QMetaType::Void,

       0        // eod
};

void Board::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Board *_t = static_cast<Board *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->started(); break;
        case 1: _t->finished((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->optimizingStarted(); break;
        case 3: _t->optimizingFinished(); break;
        case 4: _t->pauseAvailable((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->setShowMissedWords((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->setShowMaximumScore((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 7: _t->setShowWordCounts((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->updateScoreColor(); break;
        case 9: _t->gameStarted(); break;
        case 10: _t->clearGuess(); break;
        case 11: _t->guess(); break;
        case 12: _t->guessChanged(); break;
        case 13: _t->finish(); break;
        case 14: _t->wordSelected(); break;
        case 15: _t->letterClicked((*reinterpret_cast< Letter*(*)>(_a[1]))); break;
        case 16: _t->showMaximumWords(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (Board::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::started)) {
                *result = 0;
            }
        }
        {
            typedef void (Board::*_t)(int );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::finished)) {
                *result = 1;
            }
        }
        {
            typedef void (Board::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::optimizingStarted)) {
                *result = 2;
            }
        }
        {
            typedef void (Board::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::optimizingFinished)) {
                *result = 3;
            }
        }
        {
            typedef void (Board::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&Board::pauseAvailable)) {
                *result = 4;
            }
        }
    }
}

const QMetaObject Board::staticMetaObject = {
    { &QWidget::staticMetaObject, qt_meta_stringdata_Board.data,
      qt_meta_data_Board,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *Board::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Board::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_Board.stringdata0))
        return static_cast<void*>(const_cast< Board*>(this));
    return QWidget::qt_metacast(_clname);
}

int Board::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void Board::started()
{
    QMetaObject::activate(this, &staticMetaObject, 0, Q_NULLPTR);
}

// SIGNAL 1
void Board::finished(int _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Board::optimizingStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 2, Q_NULLPTR);
}

// SIGNAL 3
void Board::optimizingFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 3, Q_NULLPTR);
}

// SIGNAL 4
void Board::pauseAvailable(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_END_MOC_NAMESPACE
